/*

  a DIVERSE DSO to set the LOD scale

  John Kelso, 5/04, kelso@nist.gov

  the is gotten from a float in the dtk shared memory file "lodScale"

 */

#include <dpf.h>
#include <dtk/dtkDSO_loader.h>

class lodScale : public dtkAugment {
public:
  lodScale(dpf *app) ;
  ~lodScale(void) ;
  int postConfig(void) ;
  int postFrame(void) ;

private:
  dpfDisplay *display ;
  pfChannel *chan ;
  u_int64_t writeCount ;
  dtkSharedMem *shm ;
} ;

////////////////////////////////////////////////////////////////////////
lodScale::lodScale(dpf *app) :
  dtkAugment("lodScale") {

  display = app->display() ;

  chan = NULL ;

  //get DTK shared memory
  // use existing segment, or create if it doesn't exist
  shm = new dtkSharedMem(sizeof(float), "lodScale", DTK_GET) ;
  if (!shm || shm->isInvalid()) {
    dtkMsg.add(DTKMSG_ERROR,"lodScale::lodScale() failed: "
	       " unable to create or get shared memory\n") ;
    return ;
  } 
  
  if(shm->writeCount(1)) return ;
  writeCount = shm->getWriteCount() ;
  shm->setAutomaticByteSwapping(sizeof(float)) ;
  
  setDescription("sets LOD scale") ;
  validate() ;
} ;

////////////////////////////////////////////////////////////////////////
lodScale::~lodScale(void) {
}

////////////////////////////////////////////////////////////////////////
int lodScale::postConfig(void) {
  
  if (!chan)
    chan = display->first()->first()->first()->pfchan(0) ;
  
  return dtkAugment::REMOVE_CALLBACK ; 
}

////////////////////////////////////////////////////////////////////////
int lodScale::postFrame(void) {
  
  float scale ;

  if (shm->read(&scale)) {
    dtkMsg.add(DTKMSG_ERROR,"lodScale::postFrame() failed: "
	       " unable to read shared memory segment %s\n","lodScale") ;
    return dtkAugment::ERROR_ ;
  }
  
  u_int64_t currentWriteCount = shm->getWriteCount() ;
  
  if (currentWriteCount != writeCount) {
    writeCount = currentWriteCount ;
#if 0
    printf("writing, count = %lld        \r", writeCount) ;  fflush(stdout) ;
#endif
    if(shm->differentByteOrder())
      dtk_swapBytes(&scale, sizeof(float)) ;
    
#if 0
    printf("scale = %3.3f\n", scale) ;	   
#endif
    chan->setLODAttr(PFLOD_SCALE, scale) ;
  }
  
  return dtkAugment::CONTINUE ; 
}


/*************** DTK C++ DSO loader/unloader functions ***********/
/*
 * All DTK DSO files are required to declare these two functions.
 * These function are called by the loading program to get your
 * C++ objects loaded.
 *
 *****************************************************************/

static dtkAugment *dtkDSO_loader(dtkManager *manager, void *p) {
  dpf *app = static_cast<dpf *>(manager) ;
  return new lodScale(app) ;
}

static int dtkDSO_unloader(dtkAugment *augment) {
  delete augment ;
  return DTKDSO_UNLOAD_CONTINUE ;
}
